#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXSTRINGLEN 256

char * CalculateBarcodeEan13(char * BarcodeNumberStr, char * PrintString)
{
   /* Function to calculate the Barcode
      Return the result string.
      Note: There could be some Printing information already stored in the PrintString!
   */

   if (strlen(BarcodeNumberStr) != 12)          /* Has to be 12 ! */
      strcat(PrintString, "BARCODELENGTH NOT 12 DIGITS!!");
   else
   {
      int i, Ende = 0, VerifySum = 0;
      for (i = 0; (i < 12) && (! Ende); i ++)
         if (strchr("1234567890\0", BarcodeNumberStr[i]) == NULL)
            Ende = 1;
      if (Ende)
         strcat(PrintString, "INVALIDE DIGIT FOUND!!");
      else
      {
         char CountryPositionStr[7];
         int Pos;

         /* Ok, now lets get the font selections sequence. We will use the
            secondary font so the primary will be unchanged.
            The pointsize of the font will be 20 points
         */
         strcat(PrintString, ")10Q)s1p20vsb10101T");

         /* Printing with numbers below the code, you want to add the country digit in front
            The charaters A..J  Print the country number!
         */
         Pos = (int) strlen(PrintString);
         PrintString[Pos++] = (unsigned char) (*BarcodeNumberStr - 48 + 'A');

         /* get the Contry positioning key */
         switch (*BarcodeNumberStr)
         {
            case '0' : strcpy(CountryPositionStr, "000000");
                       break;
            case '1' : strcpy(CountryPositionStr, "001011");
                       break;
            case '2' : strcpy(CountryPositionStr, "001101");
                       break;
            case '3' : strcpy(CountryPositionStr, "001110");
                       break;
            case '4' : strcpy(CountryPositionStr, "010011");
                       break;
            case '5' : strcpy(CountryPositionStr, "011001");
                       break;
            case '6' : strcpy(CountryPositionStr, "011100");
                       break;
            case '7' : strcpy(CountryPositionStr, "010101");
                       break;
            case '8' : strcpy(CountryPositionStr, "010110");
                       break;
            case '9' : strcpy(CountryPositionStr, "011010");
                       break;
         }

         /* Ok, now we can create the barcode
            Note if CountryPosition is a 1 we have to add 32 to get to the
                    range P..Y for the fist 6 Bytes,
                 for the last bytes we have to add 64
         */
         PrintString[Pos++] = 'b';   /* Start Byte */
         for (i = 0; i < 6; i++)
         {
            PrintString[Pos] = BarcodeNumberStr[i+1];
            if (CountryPositionStr[i] != '0')
               PrintString[Pos] += 32;   /* range P..Y !!! */
            Pos ++;
         }
         PrintString[Pos++] = 'm';   /* Middle Byte */
         for (i = 7; i < 12; i++)
         {
            PrintString[Pos] = BarcodeNumberStr[i];
            PrintString[Pos] += 64;   /* range p..y !! */
            Pos++;
         }

         /* Now we have to calculate the verify digit. */
         for (i = 0; i < 12; i++)
         {
            VerifySum += BarcodeNumberStr[i] - 48 + (BarcodeNumberStr[i + 1] - 48) * 3;
            i++;
         }
         i = 0;
         while (VerifySum++ % 10)
            i ++;
         PrintString[Pos] = (unsigned char) (i + 112); /* range p..y !! */
         PrintString[++Pos] = 0x0;

         /* finally we must add the end byte and return to the primary font*/
         strcat(PrintString, "e");
      }
   }
   return(PrintString);
}

char * GetPos(float Position, char * BufferStr, char EndStr)
{
   /* The offset Position has to be multiplied with 28.34 to get
      the 1/720 inch resolution
   */
   sprintf(BufferStr, "&a%u%c", (unsigned int) (Position * 28.34), EndStr);
   return(BufferStr);
}

char * CalculatePosBarcodeEan13(char * BarcodeNumber, char * PrintString, float xPos, float yPos)
{
   char BufferStr[20];

   strcat(PrintString, "&fS");   /* Store actual printing Position in printer */
   strcat(PrintString, GetPos(xPos, BufferStr, 'H'));           /* Add &a###H */
   strcat(PrintString, GetPos(yPos, BufferStr, 'V'));           /* Add &a###V */
   CalculateBarcodeEan13(BarcodeNumber, PrintString);
   strcat(PrintString, "&f1S");  /* recall printing Position in printer */
   return(PrintString);
}

void main(int argc,char *argv[])
{
   /* Programm for printing barcode 2of5 interleaved.
      Functionality:  The user will enter the barcode number
   */
   char BarcodeNumber[13];
   char PrintString[MAXSTRINGLEN];

   *PrintString = 0x0;
   printf("\n\nPlease enter the barcode number: ");
   scanf("%s", &BarcodeNumber);
   printf("Print string: %s\n\n", CalculateBarcodeEan13(BarcodeNumber, PrintString));

   {  /* second example
         Is just teh same as the first one, in addition you will enter a
         special printing position. The printing of the barcode will not
         change the printing font and the actual position!
      */
      float xPos, yPos;

      printf("Position in mm from the left paper margin: ");
      scanf("%f", &xPos);
      printf("Position in mm from the upper paper margin: ");
      scanf("%f", &yPos);

      /* This is an additional reset sequence. We have to add it for this
         example, because we have to set the printers physical top print position
         to zero! See rechnical reference manual for further explanations.
      */
      strcpy(PrintString, "[E&lE]");
      printf("Print string: %s\n\n", CalculatePosBarcodeEan13(BarcodeNumber, PrintString, xPos, yPos));
   }
   exit(0);
}
